/*
 *		L I S T _ A d d
 *
 *	This function is called when it is required
 *	to insert a new node in order in any list.
 *
 *	If it is called on an unordered list we
 *	simply revert to adding at the end.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *	   Email:  dave.walker@icl.com
 *
 *	   Permission to copy and/or distribute granted under the
 *	   following conditions:
 *
 *	   1). This notice must remain intact.
 *	   2). The author is not responsible for the consequences of use
 *		   this software, no matter how awful, even if they
 *		   arise from defects in it.
 *	   3). Altered version must not be represented as being the
 *		   original software.
 *
 *	AMENDMENT HISTORY
 *	~~~~~~~~~~~~~~~~~
 *	22 May 96	DJW   - First version
 *
 *	31 May 96	DJW   - Amended for data structure changes to support
 *						the 'Clone' method.
 *
 *	30 Jun 96	DJW   - Amended to always add at start for LIFO, rather
 *						than at end as for other unordered types.
 */

#include <liblist.h>
#include "liblistx.h"

int 	LIST_Add (list_t list, node_t node)
{
	_listmaster_t  listmaster;
	listclass_t    listclass;
	node_t libnode;

	LIBDEBUG ("LIST_Add", \
		   "'list' parameter is not a valid list instance", \
		   _LIST_ListCheck(list))
	LIBDEBUG ("LIST_Add", \
		   "'node' parameter is not a valid member of this list type", \
		   _LIST_NodeCheck(list,node))
	LIBDEBUG ("LIST_Add", \
		   "node is already a member of this list", \
		   LIST_Index(list,node) == 0)

	listclass = list->listclass;
	listmaster = listclass->master;

	node = _LIST_REALNODE(list,node);

	if ((listclass->compare != NULL)
	 || LIST_Embedded(list)) 
	{
		/*
		 *	If this is an ordered list, or a
		 *	we are handlimg an embedded list then
		 *	use the list specific add routine.
		 */
		return (listmaster->add)(list, node);
	}

	/*
	 *	If this is an unordered list, then
	 *	we always insert at the end of the list
	 *	unless it is a LIFO when we always
	 *	insert at the start.
	 */
	switch (LIST_Type(list))
	{
	case LIST_TYPE_LIFO:
			libnode = NULL;
			break;
	default:
			libnode = listmaster->last(list);
			break;
	}
	if (libnode != NULL)
	{
		libnode = _LIST_REALNODE(list, libnode);
	}
	return	(listmaster->after)(list, node, libnode);
}
