/*
 *          _ L I S T _ F i f o A f t e r
 *
 *  This routine adds a node on a list that
 *  DOES maintain a pointer to the last node
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  23 May 96   DJW   - First version
 *
 *  28 May 96   DJW   - Updated 'remove' to allow for the user providing
 *                      a node to remove who that not on the list!
 */


#include <liblist.h>
#include "liblistx.h"


int     _LIST_FifoAfter (list_t list, node_t newnode, node_t oldnode)
{
	/*
	 *	When LIBTEST is defined, assign parameters
	 *	to local variables to help with debugging
	 */
#ifdef LIBTEST
	register _fifoh_t fifolist = (_fifoh_t)list;
	register _fifo_t  fifonewnode = (_fifo_t)newnode;
	register _fifo_t  fifooldnode = (_fifo_t)oldnode;
#else
#define fifolist ((_fifoh_t)list)
#define fifonewnode ((_fifo_t)newnode)
#define fifooldnode ((_fifo_t)oldnode)
#endif

    int     reply;

	/*
	 *	Are we trying to insert at END
	 *	(which is commonest case) as we
	 *	can optimise this
	 */
	if (fifooldnode != NULL 
	&&  fifooldnode == fifolist->last)
	{
		LIBASSERT (fifooldnode->next == NULL);
        fifonewnode->next = NULL;
        fifooldnode->next = fifonewnode;
		reply = LIST_ERROR_NONE;
	}
	else
	{
		/*
		 *	Use same logic as for LIFO if elsewhere in list 
		 */
	    reply = _LIST_LifoAfter(list, newnode, oldnode);
	}
	/*
	 *	Now update the LAST pointer
	 */
	if (fifolist->last == NULL  || fifolist->last == fifooldnode)
	{
	    fifolist->last = fifonewnode;
	}
    return (reply);
}

