/*
 *		l i f o _ c
 *
 *	This contains routines that are specific to the
 *	implementation of LIFO lists (stacks)
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *	   Permission to copy and/or distribute granted under the
 *	   following conditions:
 *
 *	   1). This notice must remain intact.
 *	   2). The author is not responsible for the consequences of use
 *		   this software, no matter how awful, even if they
 *		   arise from defects in it.
 *	   3). Altered version must not be represented as being the
 *		   original software.
 *
 *
 *	AMENDMENT HISTORY
 *	~~~~~~~~~~~~~~~~~
 *	12 May 96	DJW   - First version
 */

#include <liblist.h>
#include "liblistx.h"


/*========================================================================
 *
 *	This is the list of vectored functions
 *	that are used to support this list type.
 *
 *------------------------------------------------------------------------*/
static struct _listmaster master =
		{
		&_LIST_ListCount,
		&_LIST_ListFirst,
		&_LIST_LifoLast,
		&_LIST_ListNext,
		&_LIST_LifoPrevious,
		&_LIST_CloneList,
		&_LIST_Destroy,
		&_LIST_NewNode,
		&_LIST_ListAdd,
		&_LIST_LifoAfter,
		&_LIST_FifoRemove,
		&_LIST_ListFind,
		LIST_TYPE_LIFO,
#ifdef LISTDEBUG
		&_LISTlifo,
		"LIFO List (stack)"
#endif /* LISTDEBUG */
		};

/*
 *	This is the default settings for a new
 *	object of this type.
 */
struct _listclass _LISTlifo =
		{
		&master,					/* List of supported functions */
		sizeof(struct _lifoh),		/* Size of the list header */
		sizeof(struct _lifo),		/* Size of the node header */
		0,							/* Size of node data */
		0,							/* Clone count */
		LIST_DUPLICATES_NA,			/* Duplicate Control */
		NULL,						/* User node Initialise */
		NULL,						/* User node Destroy */
		NULL,						/* user node Compare */
		};
