/*
 *      _ L I S T _ L i s t R e m o v e
 *
 *
 *  This function is an internal support routine that
 *  is used to find a node in a linked list and then
 *	remove it from that list.
 *	It only updates the 'next' pointer, so any remaining
 *	pointers need to be maitained by the calling code.
 *
 *	Return values:
 *		-ve		Error code
 *		0		Success, but list now empty
 *		+ve		Success, and this is address of previous node.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  28 May 96   DJW   - Updated 'remove' to allow for the user providing
 *                      a member to remove who is not on the list!
 *
 *  09 May 99   DJW   - Moved to this new rotuine code that was previously common
 *						to the _LIST_FifoRemove and _LIST_LifoRemove routines
 *						(which now call this one).
 */


#include <liblist.h>
#include "liblistx.h"


long  _LIST_ListRemove (list_t list, node_t node)

{
#ifdef LIBTEST
	_lifoh_t	lifolist = (_lifoh_t)list;
	_lifo_t		lifonode = (_lifo_t)node;
#else
#define lifolist   ((_lifoh_t)list)
#define lifonode  ((_lifo_t)node)
#endif
	_lifo_t listnode = NULL;

    if (lifonode == NULL)
	{
		return LIST_ERROR_BADNODE;
	}
    /*
     *  If this is the first node, then
     *  we have an easy life!
     */
    if (lifonode == lifolist->nodes)
    {
        lifolist->nodes = lifonode->next;
    }
    else
    {
        /*
         *  We need to scan the whole list in sequence
         */
        for (listnode = lifolist->nodes ; listnode != NULL ; listnode = listnode->next)
        {
            if (listnode->next == lifonode)
            {
                break;
            }
        }
        /*

         *  We had better check that we found it!
         *  The user could give us a member not attached
         *  (or a NULL value which has the same effect).
         */
        if (listnode == NULL)
        {
            return LIST_ERROR_NOTFOUND;
        }
        listnode->next = lifonode->next;
    }
	/*
	 *	Remove link from current node
	 */
    lifonode->next = NULL;
    return (long)listnode;
}

