/*
 *		L I S T _ N e w B e f o r e
 *
 *	This function is called when it is required
 *	to insert a new node before a specifed node.
 *
 *	We have actually only really implemented inserting
 *	nodes after a given node in the support routines,
 *	so we do the work to effectively change this into
 *	a List_After() type routine.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *	   Email:  dave.walker@icl.com
 *
 *	   Permission to copy and/or distribute granted under the
 *	   following conditions:
 *
 *	   1). This notice must remain intact.
 *	   2). The author is not responsible for the consequences of use
 *		   this software, no matter how awful, even if they
 *		   arise from defects in it.
 *	   3). Altered version must not be represented as being the
 *		   original software.
 *
 *
 *	AMENDMENT HISTORY
 *	~~~~~~~~~~~~~~~~~
 *	22 May 96	DJW   - First version
 *
 *	09 May 99	DJW   - Changed to use new support routine _LIST_NewGetNode
 *						Fixed problem where error code could be corrupted when operation failed.
 */


#include <liblist.h>
#include "liblistx.h"

/*
 *	This function combines the LIST_New() functionality
 *	with that of the LIST_Before() method in one call.
 */
node_t	 LIST_NewBefore (list_t list, node_t oldnode, ...)
{
	node_t	newnode;
	int 	reply;
	va_list ap;

#ifdef LIBDEBUG
	LIBDEBUG ("LIST_NewBefore", \
		   "'list' parameter is not a valid list instance", \
		   _LIST_ListCheck(list))

	LIBDEBUG ("LIST_NewBefore", \
		   "'oldnode' is not a valid member of this list", \
		   _LIST_NodeCheck(list,oldnode))
	LIBDEBUG  ("LIST_NewBefore", \
		   "'oldnode' does not exist in this list", \
		   LIST_Index(list,oldnode))

#endif /* LIBDEBUG */

	/*
	 *	Create a new node
	 *	(passing on variable parameter list)
	 */
	va_start (ap, oldnode);
	newnode = _LIST_NewGetNode(list, ap);
	va_end(ap);
	if (newnode != NULL)
	{
		reply = LIST_Before(list,newnode,oldnode);
		if (reply != LIST_ERROR_NONE)
		{
#ifdef LIBTEST
			LIBASSERT (LIST_FreeNode(list,&newnode) == LIST_ERROR_NONE)
#else
			(void)LIST_FreeNode(list,&newnode);
#endif
			errno = reply;
			return (NULL);
		}
	}
	return (newnode);
}
