/*
 *      L I S T _ N e w I n s e r t
 *
 *  This function is called when it is required
 *  to insert a new node at the beginning of a list.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 *
 *  03 May 99   DJW   - Corrected problem where absence of an 'initnode'
 *                      routine return LIST_ERROR_ORDERED instead of the
 *                      correct code of LIST_ERROR_BADINIT.
 *
 *	09 May 99	DJW	  -	Changed to use new support routine _LIST_NewGetNode
 *						Fixed problem where error code could be corrupted when operation failed.
 */

#include <liblist.h>
#include "liblistx.h"


/*
 *  This function combines the LIST_New() functionality
 *  with that of the LIST_Insert() method in one call.
 */
node_t    LIST_NewInsert (list_t list, ...)
{
    node_t  newnode;
    int     reply;
    va_list ap;

    LIBDEBUG ("LIST_NewInsert", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListCheck(list))

    /*
     *  Create a new node
     *  (passing on variable parameter list)
     */
    va_start (ap, list);
    newnode = _LIST_NewGetNode(list, ap);
    va_end(ap);
    if (newnode != NULL)
    {
	    reply = LIST_Insert(list,newnode);
		if (reply != 0)
	    {
#ifdef LIBTEST
		    LIBASSERT (LIST_FreeNode(list,&newnode) == LIST_ERROR_NONE)
#else
	        (void)LIST_FreeNode(list,&newnode);
#endif /* LIBTEST */
		    errno = reply;
			newnode = NULL;
		}
	}
    return (newnode);
}


